/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TComboBox - Type ComboBox

Hirarchie:
TObject -> TComponent -> TComboBox
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TComboBox reprsente un menu droulant.
 @since Release 1.0
*/

//Structure de l'objet de Type ComboBox extension de TComponent
typedef struct ComboBox {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct ComboBox*	(*clone)		(struct ComboBox*);
	BOOL							(*equals)		(struct Component*, struct Component*);
	void							(*finalize)	(struct ComboBox*);
	TString* 					(*toString) (struct Component*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)			(struct ComboBox*);
	void							(*setBounds)	(struct Component*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Component*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	TVector* items;			//Les items du combo
	TObject* selected;  //L'item slectionn
	TCursor* cursor;		//Le curseur utilis par le combo
	void							(*use)					(struct ComboBox*);
	void							(*setSelected)	(struct ComboBox*, TObject*);
} TComboBox;

//Dclaration des constructeurs
TComboBox* TComboBox0();

//Dclaration des mthodes
TComboBox*	TComboBox_clone				(TComboBox*);
void				TComboBox_finalize		(TComboBox*);
void				TComboBox_print				(TComboBox*);
void				TComboBox_use					(TComboBox*);
void				TComboBox_setSelected	(TComboBox*, TObject*);

/**
 * Ce constructeur instancie un objet de Type ComboBox.
 @since Release 1.0
*/
TComboBox* TComboBox0 () {
	TComboBox *this = (TComboBox*)malloc(sizeof(TComboBox));
	
	this->class  = "TComboBox";
	this->super  = "TComponent";
	
	this->height   	= 15;
	this->visible 	= FALSE;
	this->width 	 	= 65;
	this->x				 	= 0;
	this->y				 	= 0;
	this->img			 	= (BITMAP*)malloc(sizeof(BITMAP));
	this->items			= TVector0();
	this->selected	= NULL;
	this->cursor 		= TCursor0();
	
	this->clone 	 = TComboBox_clone;
	this->equals 	 = TComponent_equals;
	this->finalize = TComboBox_finalize;
	this->toString = TComponent_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TComboBox_print;
	this->setBounds  = TComponent_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TComponent_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;

	this->use				 	= TComboBox_use;
	this->setSelected	= TComboBox_setSelected;

	return this;
}

/**
 * Cette mthode retourne un objet de Type ComboBox qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TComboBox* TComboBox_clone(TComboBox* this) {

	TComboBox *clone;
	
	clone = TComboBox0();
	
	clone->height 	= this->height;
	clone->visible	= FALSE;
	clone->width		= this->width;
	clone->x				= this->x;
	clone->y				= this->y;
	clone->items		= this->items->clone(this->items);
	clone->selected	= this->selected;
	clone->cursor		= this->cursor;
	
	return clone;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TComboBox_finalize (TComboBox* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		this->items->finalize(this->items);
		free(this->img);
		free(this);	
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode affiche le composant  l'cran.
 @since Release 1.0
*/
void TComboBox_print(TComboBox* this) {
	
	int x0 = this->x;
	int y0 = this->y;
	int x1 = x0 + this->width - 1;
	int y1 = y0 + this->height - 1;
	int i;
	TString* caption;
	
	if (this->selected != NULL) {
		caption = this->selected->toString(this->selected);
	}
	else {
		caption = s("");
	}
	
	for (i=y0; i<y1; i++) {
		DrawLine(x0, i, x1, i, A_REVERSE);
	}
	
	DrawLine(x0, y0, x1, y0, A_NORMAL);
	DrawLine(x0, y1, x1, y1, A_NORMAL);
	DrawLine(x0, (y0+1), x0, (y1-1), A_NORMAL);
	DrawLine(x1, (y0+1), x1, (y1-1), A_NORMAL);
	
	DrawStr((x0+2), (y0+2), caption->string, A_NORMAL);
	
	caption->finalize(caption);
	
	DrawLine(x1 - 14, y0, x1 - 14, y1, A_NORMAL);
	
	Sprite8(x1 - 10, y0 + 3, 8, Cursor_DOWN_ARROW_CURSOR, LCD_MEM, SPRT_OR);
}

/**
 * Cette mthode permet  l'utilisateur d'utiliser le ComboBox.
 @since Release 1.0
*/
void TComboBox_use(TComboBox* this) {
	
	int x0 = this->x;
	int y0 = this->y;
	int x1 = x0 + this->width - 1;
	int y1 = y0 + this->height - 1;
	int i;
	BOOL exit = FALSE;
	TLabel* items[this->items->size];
	TObject *obj;
	TString *str;
	TPanel *p = TPanel0();
	
	if (this->items->size <= 0) {
		return;
	}
	
	for (i = 0; i < this->items->size; i++) {
		obj = (TObject*)this->items->elementAt(this->items, i);
		str = (obj)->toString(obj);
		items[i] = TLabel1(str);
		str->finalize(str);
		if (i > 0) {
			items[i]->move(items[i], x0 + 1, items[i-1]->y + items[i-1]->height + 1);
		}
		else {
			items[i]->move(items[i], x0 + 1, y1 + 1);
		}
		items[i]->setWidth(items[i], this->width - 1);
	}
	
	p->setBounds(p, x0, y1, x1, y1 + ((items[0]->height + 1) * this->items->size + 1));
	
	p->setVisible(p, TRUE);
	
	for (i = 0; i < this->items->size; i++) {
		items[i]->setVisible(items[i], TRUE);
	}
	
	while (!exit) {
		this->cursor->use(this->cursor);

		for (i = 0; i < this->items->size; i++) {
			if (this->cursor->isOver(this->cursor, items[i])) {
				this->setVisible(this, FALSE);
				this->selected = (TObject*)this->items->elementAt(this->items, i);
				this->setVisible(this, TRUE);
				exit = TRUE;
			}
		}
		
		if (i >= this->items->size) {
			exit = TRUE;
		}
	}
	
	for (i = 0; i < this->items->size; i++) {
		items[i]->finalize(items[i]);
	}
	
	p->finalize(p);
}

/**
 * Cette mthode change l'item slectionn.
 @param selected L'item slectionn
 @since Release 1.0
*/
void TComboBox_setSelected(TComboBox* this, TObject* selected) {

	BOOL visible = this->visible;
	
	if (visible) {
		TComponent_setVisible(this, FALSE);
	}
	
	this->selected = selected;
	
	if (visible) {
		TComponent_setVisible(this, TRUE);
	}
}
