/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TComponent - Type Component

Hirarchie:
TObject -> TComponent
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TComponent est la racine de la hirarchie des composants visuels.  Elle dfinie
 * les proprits communes  tous les objets visuels.
 @since Release 1.0
*/

//Structure de l'objet de Type Component extension de TObject
typedef struct Component {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct Component*	(*clone)		(struct Component*);
	BOOL							(*equals)		(struct Component*, struct Component*);
	void							(*finalize)	(struct Component*);
	TString* 					(*toString) (struct Component*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)			(struct Component*);
	void							(*setBounds)	(struct Component*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Component*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
} TComponent;

//Dclaration des constructeurs
TComponent* TComponent0();

//Dclaration des mthodes
TComponent*	TComponent_clone			(TComponent*);
BOOL				TComponent_equals			(TComponent*, TComponent*);
void				TComponent_finalize		(TComponent*);
void				TComponent_move				(TComponent*, int, int);
void				TComponent_print			(TComponent*);
void				TComponent_setBounds	(TComponent*, int, int, int, int);
void				TComponent_setHeight	(TComponent*, int);
void				TComponent_setVisible	(TComponent*, BOOL);
void				TComponent_setWidth		(TComponent*, int);
void				TComponent_setX				(TComponent*, int);
void				TComponent_setY				(TComponent*, int);
TString*		TComponent_toString		(TComponent*);

/**
 * Ce constructeur instancie un objet de Type Component.
 @since Release 1.0
*/
TComponent* TComponent0 () {
	TComponent *this = (TComponent*)malloc(sizeof(TComponent));
	
	this->class  = "TComponent";
	this->super  = "TObject";
	
	this->height 	= 0;
	this->visible = FALSE;
	this->width 	= 0;
	this->x				= 0;
	this->y				= 0;
	this->img			= (BITMAP*)malloc(sizeof(BITMAP));
	
	this->clone 	 = TComponent_clone;
	this->equals 	 = TComponent_equals;
	this->finalize = TComponent_finalize;
	this->toString = TComponent_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TComponent_print;
	this->setBounds  = TComponent_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TComponent_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;

	return this;
}

/**
 * Cette mthode retourne un objet de Type Component qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TComponent* TComponent_clone(TComponent* this) {

	TComponent *clone;
	
	clone = TComponent0();
	
	clone->height 	= this->height;
	clone->visible	= FALSE;
	clone->width		= this->width;
	clone->x				= this->x;
	clone->y				= this->y;
	
	return clone;
}

/**
 * Cette mthode compare l'objet  un autre objet pour tablir une galit.
 * Dans le cas d'un objet de Type Component, une comparaison est tablie
 * entre les proprits.
 @param component0 L'objet  comparer.
 @return result - Vrai si les deux objets sont gaux.
 @since Release 1.0
*/
BOOL TComponent_equals (TComponent* this, TComponent* component0) {
	
	BOOL result;
	
	if ( this->height == component0->height &&
			 this->visible == component0->visible &&
			 this->width == component0->width &&
			 this->x == component0->x &&
			 this->y == component0->y
			) {
		result = TRUE;
	}
	else {
		result = FALSE;
	}
	
	return result;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TComponent_finalize (TComponent* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		free(this->img);
		free(this);	
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode affiche le composant  l'cran.
 @since Release 1.0
*/
void TComponent_print(TComponent* this) {
	
}

/**
 * Cette mthode dplace le composant vers de nouvelles coordones.
 @param x La position horizontale du composant
 @param y La position verticale du composant 
 @since Release 1.0
*/
void TComponent_move(TComponent* this, int x, int y) {
	
	this->setBounds(this, x, y, x + this->width - 1, y + this->height - 1);
}

/**
 * Cette mthode change la visibilit du composant.
 @param visible La visibilit du composant
 @since Release 1.0
*/
void TComponent_setVisible(TComponent* this, BOOL visible) {

	SCR_RECT rec;
	
	if (visible) {
		if (!this->visible) {
			rec.xy.x0 = this->x;
			rec.xy.y0 = this->y;
			rec.xy.x1 = this->width - 1 + this->x;
			rec.xy.y1 = this->height - 1 + this->y;
			free(this->img);
			this->img = (BITMAP*)malloc(BitmapSize(&rec));
			BitmapGet (&rec, this->img);
			this->print(this);
		}
	}
	else {
		if (this->visible) {
			BitmapPut (this->x, this->y, this->img, ScrRect, A_REPLACE);
		}
	}

	this->visible = visible;
}

/**
 * Cette mthode permet de spcifier l'espace occup par le composant.
 @param x0 La position horizontale du coin supprieur gauche
 @param y0 La position verticale du coin supprieur gauche
 @param x1 La position horizontale du coin infrieur droit
 @param y1 La position verticale du coin infrieur droit
 @since Release 1.0
*/
void TComponent_setBounds(TComponent* this, int x0, int y0, int x1, int y1) {
	
	BOOL visible = this->visible;
	
	if (visible) {
		TComponent_setVisible(this, FALSE);
	}
	
	this->x = x0;
	this->y = y0;
	this->width = x1 - x0 + 1;
	this->height = y1 - y0 + 1;
	
	if (visible) {
		TComponent_setVisible(this, TRUE);
	}
}

/**
 * Cette mthode change la hauteur du composant
 @param height La hauteur du composant
 @since Release 1.0
*/
void TComponent_setHeight(TComponent* this, int height) {

	this->setBounds(this, this->x, this->y, this->x + this->width - 1, this->y + height - 1);
}

/**
 * Cette mthode change la largeur du composant
 @param width La largeur du composant
 @since Release 1.0
*/
void TComponent_setWidth(TComponent* this, int width) {

	this->setBounds(this, this->x, this->y, this->x + width - 1, this->y + this->height - 1);
}

/**
 * Cette mthode change la position horizontale du composant
 @param x La position horizontale du composant
 @since Release 1.0
*/
void TComponent_setX(TComponent* this, int x) {

	this->move(this, x, this->y);
}

/**
 * Cette mthode change la position verticale du composant
 @param y La position verticale du composant
 @since Release 1.0
*/
void TComponent_setY(TComponent* this, int y) {

	this->move(this, this->x, y);
}

/**
 * Cette mthode retourne une chane de caractres reprsentant l'objet.
 * Pour la classe TComponent, il s'agit des coordonnes (x, y) suivies des
 * dimensions (largeur, hauteur).
 @return string - La chane de caractre reprsentant l'objet.
 @since Release 1.0
*/
TString* TComponent_toString	(TComponent* this) {
	
	TString* string;
	
	string = TString2(7, Integer_toString(this->x), s("@"),
											 Integer_toString(this->y), s("@"),
											 Integer_toString(this->width), s("@"),
											 Integer_toString(this->height));
	
	return string;
}
