/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TContainer - Type Container

Hirarchie:
TObject -> TComponent -> TContainer
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TContainer reprsente un contenant d'objets visuels.
 @since Release 1.0
*/

//Structure de l'objet de Type Container extension de TComponent
typedef struct Container {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct Container*	(*clone)		(struct Container*);
	BOOL					(*equals)				(struct Component*, struct Component*);
	void					(*finalize)			(struct Container*);
	TString* 			(*toString) 		(struct Component*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)			(struct Component*);
	void							(*setBounds)	(struct Container*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Container*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	TVector* contentPane;	//Le contenu du Container
	BOOL* visibility;			//La visibilit des composants
	void							(*bringOn)		(struct Container*, TComponent*);
	void							(*center)			(struct Container*, TComponent*);
	void							(*drag)				(struct Container*, TComponent*, TCursor*);
	void							(*add)				(struct Container*, TComponent*);
} TContainer;

//Dclaration des constructeurs
TContainer* TContainer0();

//Dclaration des mthodes
TContainer*	TContainer_clone			(TContainer*);
void				TContainer_finalize		(TContainer*);
void				TContainer_setBounds	(TContainer*, int, int, int, int);
void				TContainer_setVisible	(TContainer*, BOOL);
void				TContainer_bringOn		(TContainer*, TComponent*);
void				TContainer_center			(TContainer*, TComponent*);
void				TContainer_drag				(TContainer*, TComponent*, TCursor*);
void				TContainer_add				(TContainer*, TComponent*);

/**
 * Ce constructeur instancie un objet de Type Container.
 @since Release 1.0
*/
TContainer* TContainer0 () {
	TContainer *this = (TContainer*)malloc(sizeof(TContainer));
	
	this->class  = "TContainer";
	this->super  = "TComponent";
	
	this->height 			= 0;
	this->visible 		= FALSE;
	this->width 			= 0;
	this->x						= 0;
	this->y						= 0;
	this->img					= (BITMAP*)malloc(sizeof(BITMAP));
	this->contentPane = TVector0();
	this->visibility	= NULL;
	
	this->clone 	 = TContainer_clone;
	this->equals 	 = TComponent_equals;
	this->finalize = TContainer_finalize;
	this->toString = TComponent_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TComponent_print;
	this->setBounds  = TContainer_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TContainer_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;

	this->bringOn		 = TContainer_bringOn;
	this->center		 = TContainer_center;
	this->drag			 = TContainer_drag;
	this->add				 = TContainer_add;

	return this;
}

/**
 * Cette mthode retourne un objet de Type Container qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TContainer* TContainer_clone(TContainer* this) {

	TContainer *clone;
	int i;
	
	clone = TContainer0();
	
	clone->height 		 = this->height;
	clone->visible		 = FALSE;
	clone->width			 = this->width;
	clone->x					 = this->x;
	clone->y					 = this->y;
	clone->contentPane = this->contentPane->clone(this->contentPane);
	
	if (this->visibility != NULL)	{
		clone->visibility = malloc(this->contentPane->size * sizeof(BOOL));
		for (i = 0; i < this->contentPane->size; i++) {
			clone->visibility[i] = this->visibility[i];
		}
	}
	
	return clone;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TContainer_finalize (TContainer* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		this->contentPane->finalize(this->contentPane);
		if (this->visibility != NULL) {
			free(this->visibility);
		}
		free(this->img);
		free(this);	
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode dplace le composant vers de nouvelles coordones.
 @param x La position horizontale du composant
 @param y La position verticale du composant 
 @since Release 1.0
*/
void TContainer_setBounds(TContainer* this, int x0, int y0, int x1, int y1) {
	
	TComponent *comp;
	long i;
	
	BOOL visible = this->visible;
	
	if (visible) {
		this->setVisible(this, FALSE);
	}
	
	for (i=0; i<this->contentPane->size; i++) {
		comp = (TComponent*)this->contentPane->elementAt(this->contentPane, i);
		//comp->setBounds(comp, comp->x + x0, comp->y + y0, comp->x + x0 + comp->width, comp->y + y0 + comp->height);
		comp->move(comp, comp->x - this->x + x0, comp->y - this->y + y0);
	}
	
	TComponent_setBounds(this, x0, y0, x1, y1);
	
	if (visible) {
		this->setVisible(this, TRUE);
	}
}

/**
 * Cette mthode change la visibilit du composant.
 @param visible La visibilit du composant
 @since Release 1.0
*/
void TContainer_setVisible(TContainer* this, BOOL visible) {

	TComponent *comp;
	long i;
	
	if (this->visibility == NULL || this->visible)	{
		if (this->visibility != NULL) {
			free(this->visibility);
		}
		this->visibility = malloc(sizeof(BOOL) * this->contentPane->size);
		for (i = 0; i < this->contentPane->size; i++) {
			comp = this->contentPane->elementAt(this->contentPane, i);
			this->visibility[i] = comp->visible;
		}
	}
	
	if (visible) {
		TComponent_setVisible(this, visible);
	}
	
	for (i=0; i<this->contentPane->size; i++) {
		comp = this->contentPane->elementAt(this->contentPane, i);
		if (!visible || this->visibility[i]) {
			comp->setVisible(comp, visible);
		}
	}
	
	if (!visible) {
		TComponent_setVisible(this, visible);
	}
}

/**
 * Cette mthode permet d'ammener un composant graphique par dessus
 * les autres.
 @param component0 Le composant
 @since Release 1.0
*/
void TContainer_bringOn (TContainer* this, TComponent* component0) {
	
	TComponent *comp;
	TVector *v;
	long index;
	long i;
	BOOL visible = this->visible;
	BOOL compV;
	
	v = this->contentPane;
	
	if ((this->visibility == NULL || this->visible) /*&& visible*/)	{
		if (this->visibility != NULL) {
			free(this->visibility);
		}
		this->visibility = malloc(sizeof(BOOL) * v->size);
		for (i = 0; i < v->size; i++) {
			comp = v->elementAt(v, i);
			this->visibility[i] = comp->visible;
		}
	}
	
	index = v->lastIndexOf1(v, component0);
	
	if (index >= 0) {
		for (i = v->size - 1; (i >= index) && visible; i--) {
			comp = v->elementAt(v, i);
			comp->setVisible(comp, FALSE);
		}
		
		compV = this->visibility[index];
		
		for (i = index; i < v->size - 1; i++)
		{
			this->visibility[i] = this->visibility[i + 1];
		}
		
		this->visibility[i] = compV;
		
		v->removeElementAt(v, index);
		v->addElement(v, component0);
		
		for (i = index; (i < v->size) && visible; i++) {
			comp = v->elementAt(v, i);
			if (this->visibility[i]) {
				comp->setVisible(comp, TRUE);
			}
		}
	}
}

/**
 * Cette mthode permet de centrer un composant graphique
 @param component0 Le composant
 @since Release 1.0
*/
void TContainer_center (TContainer* this, TComponent* component0) {
	
	this->bringOn(this, component0);
	
	component0->move(component0, this->x + (this->width - component0->width) / 2, this->y + (this->height - component0->height) / 2);
}

/**
 * Cette mthode permet  l'utilisateur de dplacer un composant dans
 * le contenant.
 @param component0 Le composant
 @param cursor Le curseur li  l'action (NULL si N/A)
 @since Release 1.0
*/
void TContainer_drag (TContainer* this, TComponent* component0, TCursor* cursor) {
	
	int x0 = component0->x;
	int y0 = component0->y;
	int key = 0;
	int pasX = Cursor_DEFAULT_SPEED;
	int pasY = Cursor_DEFAULT_SPEED;
	unsigned short key_right = KEY_RIGHT;
	unsigned short key_left = KEY_LEFT;
	unsigned short key_up = KEY_UP;
	unsigned short key_down = KEY_DOWN;
	unsigned short key_up_right = key_up | key_right;
	unsigned short key_up_left = key_up | key_left;
	unsigned short key_down_right = key_down | key_right;
	unsigned short key_down_left = key_down | key_left;
	unsigned short key_enter = KEY_ENTER;
	unsigned short lcd_width = LCD_WIDTH;
	unsigned short lcd_height = LCD_HEIGHT;	
	
	if (cursor != NULL) {
		pasX = cursor->speedX;
		pasY = cursor->speedY;
	}
	
	this->bringOn(this, component0);
	
	do {
		if (kbhit()) {
			key = ngetchx();
			
			if (key == key_right) {
				x0 = x0 + pasX;
			}
			else if (key == key_left) {
				x0 = x0 - pasX;
			}
			else if (key == key_up) {
				y0 = y0 - pasY;
			}
			else if (key == key_down) {
				y0 = y0 + pasY;
			}
			else if (key == key_down_right) {
				y0 = y0 + pasY;
				x0 = x0 + pasX;
			}
			else if (key == key_down_left) {
				y0 = y0 + pasY;
				x0 = x0 - pasX;
			}
			else if (key == key_up_right) {
				y0 = y0 - pasY;
				x0 = x0 + pasX;
			}
			else if (key == key_up_left) {
				y0 = y0 - pasY;
				x0 = x0 - pasX;
			}
			if ((x0 + component0->width) > (this->x + this->width)) {
				x0 = this->x + this->width - component0->width;
			}
			
			if ((y0 + component0->height) > (this->y + this->height)) {
				y0 = this->y + this->height - component0->height;
			}
			
			if (x0 < this->x) {
				x0 = this->x;
			}
			
			if (y0 < this->y) {
				y0 = this->y;
			}
			
			component0->move(component0, x0, y0);
		}
	} while(key != KEY_ENTER);
	
	if (cursor != NULL) {
		cursor->move(cursor, component0->x, component0->y);
	}
}

/**
 * Cette mthode permet  l'utilisateur d'ajouter un composant au
 * contenant.
 @param component0 Le composant  ajouter
 @since Release 1.0
*/
void TContainer_add (TContainer* this, TComponent* component0) {

	BOOL *visibility;
	int i = 0;
	
	visibility = malloc(sizeof(BOOL) * (this->contentPane->size + 1));
	
	for (i = 0; i < this->contentPane->size; i++) {
		visibility[i] = this->visibility[i];
	}
	
	if (this->visibility != NULL) {
		free(this->visibility);
	}
	
	visibility[i] = component0->visible;

	this->visibility = visibility;

	this->contentPane->addElement(this->contentPane, component0);
}