/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TGarbageCollector - Type GarbageCollector

Hirarchie:
TObject -> TGarbageCollector
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TGarbageCollector permet de grer la suppression des objets.
 @since Release 1.0
*/

//Structure de l'objet de Type GarbageCollector extension de TObject
typedef struct GarbageCollector {
	char *class; //Le nom de la classe
	char *super; //Le nom de la superclasse
	struct GarbageCollector* 	(*clone) 		(struct GarbageCollector*);
	BOOL 											(*equals) 	(struct Object*, struct Object*);
	void 											(*finalize) (struct GarbageCollector*);
	TString* 									(*toString) (struct Object*);
	TVector* bag;
	void											(*add)			(struct GarbageCollector*, TObject*);
	void											(*collect)	(struct GarbageCollector*);
} TGarbageCollector;

//Dclaration des constructeurs
TGarbageCollector* TGarbageCollector0();

//Dclaration des mthodes
TGarbageCollector* 	TGarbageCollector_clone 		(TGarbageCollector*);
void 								TGarbageCollector_finalize	(TGarbageCollector*);
void								TGarbageCollector_collect		(TGarbageCollector*);
void								TGarbageCollector_add				(TGarbageCollector*, TObject*);


/**
 * Ce constructeur instancie un objet de Type GarbageCollector.
 @since Release 1.0
*/
TGarbageCollector* TGarbageCollector0 () {
	TGarbageCollector *this = (TGarbageCollector*)malloc(sizeof(TGarbageCollector));
	
	this->class  = "TGarbageCollector";
	this->super  = "TObject";
	
	this->bag		 = TVector0();
	
	this->clone    = TGarbageCollector_clone;
	this->equals   = TObject_equals;
	this->finalize = TGarbageCollector_finalize;
	this->toString = TObject_toString;

	this->collect = TGarbageCollector_collect;
	this->add			= TGarbageCollector_add;
	
	return this;
}

/**
 * Cette mthode retourne un objet de Type GarbageCollector qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TGarbageCollector* TGarbageCollector_clone (TGarbageCollector* this) {

	TGarbageCollector *clone;
	
	clone = TGarbageCollector0();
	
	clone->bag = this->bag->clone(this->bag);
	
	return clone;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TGarbageCollector_finalize(TGarbageCollector* this) {
		
		TRY {
			this->collect(this);
			this->bag->finalize(this->bag);
			free(this);	
		}
		ONERR {}
		ENDTRY
}

/**
 * Cette mthode supprime les objets contenus dans le GarbageCollector.
 @since Release 1.0
*/
void TGarbageCollector_collect(TGarbageCollector* this) {
		
		TObject* obj;
		int i;
		
		for (i = 0; i < this->bag->size; i++) {
			obj = (TObject*)this->bag->elementAt(this->bag, i);
			if (obj != NULL) {
				obj->finalize(obj);
			}
		}
		
		this->bag->removeAllElements(this->bag);
}

/**
 * Cette mthode ajoute un objet dans le GarbageCollector.
 @param obj L'objet  ajouter
 @since Release 1.0
*/
void TGarbageCollector_add(TGarbageCollector* this, TObject* obj) {
	
	this->bag->addElement(this->bag, obj);
}
