/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TList - Type List

Hirarchie:
TObject -> TComponent -> TList
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TList reprsente une liste.
 @since Release 1.0
*/

//Structure de l'objet de Type ComboBox extension de TComponent
typedef struct List {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct List*			(*clone)		(struct List*);
	BOOL							(*equals)		(struct Component*, struct Component*);
	void							(*finalize)	(struct List*);
	TString* 					(*toString) (struct Component*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)			(struct List*);
	void							(*setBounds)	(struct Component*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct List*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	TVector* items;							//Les items de la liste
	int index;  								//L'index de l'item slectionn
	TCursor* cursor;						//Le curseur utilis par la liste
	TGarbageCollector* collector;	//Espace mmoire utilis pour l'affichage
	void							(*use)										(struct List*);
	void							(*setSelectedIndex)				(struct List*, int);
	TObject*					(*getSelectedItem)				(struct List*);
} TList;

//Dclaration des constructeurs
TList* TList0();

//Dclaration des mthodes
TList*		TList_clone							(TList*);
void			TList_finalize					(TList*);
void			TList_print							(TList*);
void			TList_use								(TList*);
void			TList_setVisible				(TList*, BOOL);
void			TList_setSelectedIndex	(struct List*, int);
TObject*	TList_getSelectedItem		(struct List*);

/**
 * Ce constructeur instancie un objet de Type List.
 @since Release 1.0
*/
TList* TList0 () {
	TList *this = (TList*)malloc(sizeof(TList));
	
	this->class  = "TList";
	this->super  = "TComponent";
	
	this->height   	= 15;
	this->visible 	= FALSE;
	this->width 	 	= 65;
	this->x				 	= 0;
	this->y				 	= 0;
	this->img			 	= (BITMAP*)malloc(sizeof(BITMAP));
	this->items			= TVector0();
	this->index			= -1;
	this->cursor 		= TCursor0();
	this->collector		= TGarbageCollector0();
	
	this->clone 	 = TList_clone;
	this->equals 	 = TComponent_equals;
	this->finalize = TList_finalize;
	this->toString = TComponent_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TList_print;
	this->setBounds  = TComponent_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TList_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;

	this->use				 			 = TList_use;
	this->setSelectedIndex = TList_setSelectedIndex;
	this->getSelectedItem  = TList_getSelectedItem;

	return this;
}

/**
 * Cette mthode retourne un objet de Type List qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TList* TList_clone(TList* this) {

	TList *clone;
	
	clone = TList0();
	
	clone->height 	= this->height;
	clone->visible	= FALSE;
	clone->width		= this->width;
	clone->x				= this->x;
	clone->y				= this->y;
	clone->items		= this->items->clone(this->items);
	clone->index		= this->index;
	clone->cursor		= this->cursor;
	
	return clone;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TList_finalize (TList* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		this->items->finalize(this->items);
		free(this->img);
		free(this);	
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode affiche le composant  l'cran.
 @since Release 1.0
*/
void TList_print(TList* this) {
	
	int x0 = this->x;
	int y0 = this->y;
	int x1 = x0 + this->width - 1;
	int y1 = y0 + this->height - 1;
	int i;
	int n;
	TObject* obj;
	TString* caption;
	TLabel* lbl;
	
	for (i=y0; i<y1; i++) {
		DrawLine(x0, i, x1, i, A_REVERSE);
	}
	
	DrawLine(x0, y0, x1, y0, A_NORMAL);
	DrawLine(x0, y1, x1, y1, A_NORMAL);
	DrawLine(x0, (y0+1), x0, (y1-1), A_NORMAL);
	DrawLine(x1, (y0+1), x1, (y1-1), A_NORMAL);
	
	for (i = 0; i < this->items->size; i++) {
		obj = (TObject*)this->items->elementAt(this->items, i);
		caption = obj->toString(obj);
		lbl = TLabel1(caption);
		lbl->setBounds(lbl, x0 + 1, y0 + 1, x1 - 1, y0 + 1 + lbl->height);
		y0 = y0 + lbl->height;
		lbl->setVisible(lbl, TRUE);
		if (this->index == i) {
			for (n=lbl->y + 1; n<(lbl->y+lbl->height + 1); n++) {
				DrawLine(this->x + 1, n, this->x + this->width - 1, n, A_XOR);
			}
		}
		this->collector->bag->addElement(this->collector->bag, lbl);
		caption->finalize(caption);
	}
}

/**
 * Cette mthode permet  l'utilisateur d'utiliser le ComboBox.
 @since Release 1.0
*/
void TList_use(TList* this) {
	
	int i;
	TObject* obj;
	
	if (this->items->size <= 0) {
		return;
	}
	
	//sop(this->items);
	
	for (i = this->items->size - 1; i >= 0; i--) {
		obj = (TObject*)this->collector->bag->elementAt(this->collector->bag, i);
		if (this->cursor->isOver(this->cursor, obj)) {
			this->setSelectedIndex(this, i);
		}
	}
}

/**
 * Cette mthode change la visibilit du composant.
 @param visible La visibilit du composant
 @since Release 1.0
*/
void TList_setVisible(TList* this, BOOL visible) {
	
	if (this->visible) {
		this->collector->collect(this->collector);
	}
	
	TComponent_setVisible(this, visible);
}

/**
 * Cette mthode change l'item qui est selectionn dans la liste.
 @param index L'index de l'item slectionn
 @since Release 1.0
*/
void TList_setSelectedIndex(TList* this, int index) {
	
	if (index == this->index || index < 0 || index >= this->items->size) {
		return;
	}
	
	this->setVisible(this, FALSE);
	
	this->index = index;
	
	this->setVisible(this, TRUE);
}

/**
 * Cette mthode retourne l'item slectionn.
 @return obj - L'item slectionn
 @since Release 1.0
*/
TObject* TList_getSelectedItem(TList* this) {
	
	TObject* obj = NULL;
	
	if (this->index >= 0) {
		obj = (TObject*)this->items->elementAt(this->items, this->index);
	}
	
	return obj;
}
