/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TPanel - Type Panel

Hirarchie:
TObject -> TComponent -> TContainer -> TPanel
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TPanel reprsente un panneau contenant des objets visuels.
 @since Release 1.0
*/

#define Panel_COLOR_WHITE 0
#define Panel_COLOR_BLACK 1

//Structure de l'objet de Type Panel extension de TContainer
typedef struct Panel {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct Panel*	(*clone)		(struct Panel*);
	BOOL					(*equals)		(struct Component*, struct Component*);
	void					(*finalize)	(struct Panel*);
	TString* 			(*toString) (struct Component*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)			(struct Panel*);
	void							(*setBounds)	(struct Container*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Container*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	TVector* contentPane;	//Le contenu du Container
	BOOL* visibility;			//La visibilit des composants
	void							(*bringOn)		(struct Container*, TComponent*);
	void							(*center)			(struct Container*, TComponent*);
	void							(*drag)				(struct Container*, TComponent*, TCursor*);
	int color;						//La couleur de fond
	void							(*setColor)		(struct Panel*, int color);
	void							(*add)				(struct Container*, TComponent*);
} TPanel;

//Dclaration des constructeurs
TPanel* TPanel0();

//Dclaration des mthodes
TPanel* TPanel_clone		(TPanel*);
void		TPanel_finalize	(TPanel*);
void		TPanel_print		(TPanel*);
void		TPanel_setColor	(TPanel*, int);

/**
 * Ce constructeur instancie un objet de Type Container.
 @since Release 1.0
*/
TPanel* TPanel0 () {
	TPanel *this = (TPanel*)malloc(sizeof(TPanel));
	
	this->class  = "TPanel";
	this->super  = "TContainer";
	
	this->height 			= 0;
	this->visible 		= FALSE;
	this->width 			= 0;
	this->x						= 0;
	this->y						= 0;
	this->img					= (BITMAP*)malloc(sizeof(BITMAP));
	this->contentPane = TVector0();
	this->visibility	= NULL;
	this->color				= Panel_COLOR_WHITE;
	
	this->clone 	 = TPanel_clone;
	this->equals 	 = TComponent_equals;
	this->finalize = TPanel_finalize;
	this->toString = TComponent_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TPanel_print;
	this->setBounds  = TContainer_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TContainer_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;

	this->bringOn		 = TContainer_bringOn;
	this->center		 = TContainer_center;
	this->drag			 = TContainer_drag;
	this->add				 = TContainer_add;
	
	this->setColor	 = TPanel_setColor;

	return this;
}

/**
 * Cette mthode affiche le composant  l'cran.
 @since Release 1.0
*/
void TPanel_print(TPanel* this) {
	
	int x0 = this->x;
	int y0 = this->y;
	int x1 = x0 + this->width - 1;
	int y1 = y0 + this->height - 1;
	int i;
	int color;
	
	if (this->color == Panel_COLOR_WHITE) {
		color = A_REVERSE;
	}
	else {
		color = A_NORMAL;
	}
	
	for (i=y0; i<=y1; i++) {
		DrawLine(x0, i, x1, i, color);
	}
	
	if (color != Panel_COLOR_BLACK) {
		DrawLine(x0, y0, x1, y0, A_NORMAL);
		DrawLine(x0, y1, x1, y1, A_NORMAL);
		DrawLine(x0, (y0+1), x0, (y1-1), A_NORMAL);
		DrawLine(x1, (y0+1), x1, (y1-1), A_NORMAL);
	}
}

/**
 * Cette mthode retourne un objet de Type Panel qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TPanel* TPanel_clone(TPanel* this) {
	
	TPanel *clone;
	int i;
	
	clone = TPanel0();
	
	clone->height 		 = this->height;
	clone->visible		 = FALSE;
	clone->width			 = this->width;
	clone->x					 = this->x;
	clone->y					 = this->y;
	clone->contentPane = this->contentPane->clone(this->contentPane);
	clone->color			 = this->color;
	
	if (this->visibility != NULL)	{
		clone->visibility = malloc(this->contentPane->size * sizeof(BOOL));
		for (i = 0; i < this->contentPane->size; i++) {
			clone->visibility[i] = this->visibility[i];
		}
	}
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TPanel_finalize (TPanel* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		this->contentPane->finalize(this->contentPane);
		if (this->visibility != NULL) {
			free(this->visibility);
		}
		free(this->img);
		free(this);	
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode change la couleur du panel.
 @param panel La couleur (Panel_COLOR_WHITE, Panel_COLOR_BLACK)
 @since Release 1.0
*/
void TPanel_setColor(TPanel* this, int color) {

	BOOL visible = this->visible;
	
	if (visible) {
		this->setVisible(this, FALSE);
	}
	
	this->color = color;
	
	if (visible) {
		this->setVisible(this, TRUE);
	}
}
